
#include "c8051.h"
#include "queue8.h"
#include "uart0.h"

#define SYSCLK (24500000 / 4)	// SYSCLK frequency in Hz

// serial communication

void uart0_puthex4(unsigned char c)
{
	c &= 15;
	uart0_put(c < 10 ? (c + '0') : (c - 10 + 'A'));
}
void uart0_puthex8(unsigned char c)
{
	unsigned char h = c;
	h >>= 4;
	uart0_puthex4(h);
	uart0_puthex4(c);
}

void uart0_putdec(unsigned short n)
{
	unsigned char i;
	unsigned char tmp[5];
	for (i = 0; i < 5; i++) {
		tmp[i] = n % 10;
		if (n < 10) {
			i++;
			break;
		}
		n /= 10;
	}
	while (i) {
		uart0_put(tmp[--i] + '0');
	}
}

// interval timer

unsigned char timerevent;

void intr_timer2() interrupt 5
{
	TF2H = 0;					// clear Timer2 interrupt flag
	timerevent = 1;
}

//

unsigned short temp_10deg;
unsigned short temp_value;
unsigned short temp_count;

void setTMR2RL(unsigned short t)
{
	TMR2RLL = t;
	TMR2RLH = t >> 8;
}

void setTMR2(unsigned short t)
{
	TMR2L = t;
	TMR2H = t >> 8;
}

void init_temperature_sensor()
{
	REF0CN = 0x07;
	ADC0CF = 0x82;
	ADC0CN = 0x80;
	AMX0SL = 0xf8;
}

//

void main()
{
	// disable watchdog timer
	PCA0MD &= ~0x40;

	// initialize SYSCLK
	OSCICN = 0x05;
	RSTSRC = 0x04;

	// initialize port
	XBR0 = 0x04;
	XBR1 = 0x03;
	XBR2 = 0x40;
	P0MDOUT |= 0x04;

	// initialize timer
	CKCON = 0x00;
	TMOD = 0x20;

	// timer1
	TH1 = -(SYSCLK / 2 / 12 / 9600);	// 9600bps
	TL1 = 0x00;
	TR1 = 1;					// start Timer1

	// timer2
	TMR2CN = 0x00;
	setTMR2RL(-(SYSCLK / 12 / 1000));
	setTMR2(0xffff);
	ET2 = 1;
	TR2 = 1;

	//

	uart0_initialize();
	init_temperature_sensor();

	EA = 1;						// enable global interrupts

	temp_10deg = 0;
	temp_value = 0;
	temp_count = 0;

	while (1) {

		if (timerevent) {
			timerevent = 0;

			temp_value += ADC0;
			temp_count++;
			if (temp_count >= 256) {
				temp_10deg =
					(
						(unsigned long)temp_value * (1000 * 10 / 2) - (8900 * 65536 / 33 * 10)
					) / 65536;
				uart0_putdec(temp_10deg / 10);
				uart0_put('.');
				uart0_put(temp_10deg % 10 + '0');
				uart0_put('\r');
				uart0_put('\n');
				temp_value = 0;
				temp_count = 0;
			}
			ADC0CN |= 0x10;
		}
	}
}
